#    This file is part of the LinuxCNC Machine Builder, a graphical
#    configuration editor for LinuxCNC. Copyright 2013 Arvid Brodin.
#
#    Although all the code is new, this program is influenced by
#    PNCconf (Copyright 2009 Chris Morley), in turn based on stepconf,
#    a graphical configuration editor for linuxcnc (Copyright 2007 Jeff
#    Epler).
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#from halcomp.halcore import HALComp
from halcomp import halcore

class MesaHALComp(halcore.HALComp):
	# owner is the Mesa FPGA card that this component is on
	def __init__(self, index):
		halcore.HALComp.__init__(self)
		self._index = index


class PWMGen(halcore.HALComp):
	def __init__(self, username):
		halcore.HALComp.__init__(self, username)

	def setup_params(self, jointno):
		self._params = []
		self._params.append(["scale", "[AXIS_%d]_SCALE" % jointno])
		self._params.append(["output-type", ""])
		self._params.append(["pwm_frequency", ""])
		self._params.append(["pdm_frequency", ""])

	def generate_nets(self, nets):
		self._net(nets, "?", "=>", "enable")
		#self._net(nets, "?", "=>", "value")

	def is_control():
		return True

# This component is not documented in the hostmot man page?
class AnalogOut(MesaHALComp):
	def __init__(self, sserial, index):
		MesaHALComp.__init__(self, index)
		self._sserial = sserial

		self._pins["output"] = ["analogout" + str(self._index), halcore.PIN_IN]

	def setup_params(self, jointno):
		self._params = []
		self._params.append(["analogout%d-scalemax" % self._index,
				"[AXIS_%d]OUTPUT_SCALE" % jointno])
		self._params.append(["analogout%d-minlim" % self._index,
				"[AXIS_%d]OUTPUT_MIN_LIMIT" % jointno])
		self._params.append(["analogout%d-maxlim" % self._index,
				"[AXIS_%d]OUTPUT_MAX_LIMIT" % jointno])

	def get_halname(self):
		# The hostmot2 driver uses a nonstandard naming scheme
		# for these, with the component name itself used as a
		# pin, and other pins/params delimited by '-' instead
		# of '.'. Return only basename:
		return self._sserial.get_halname()


class AnalogEna(halcore.HALComp):
	def __init__(self, sserial):
		halcore.HALComp.__init__(self)
		self._sserial = sserial
		self._pins["enable"] = ["analogena", halcore.PIN_IN]

	def get_halname(self):
		# The hostmot2 driver uses a nonstandard naming scheme
		# for these, with the component name itself used as a
		# pin, and other pins/params delimited by '-' instead
		# of '.'. Return only basename:
		return self._sserial.get_halname()


class Encoder(MesaHALComp):
	def __init__(self, mesa_db, index):
		MesaHALComp.__init__(self, index)
		self._mesa_db = mesa_db

		# pin[net-name] = [pin-name, pin-dir]
		self._pins["pos-fb"] = ["position", halcore.PIN_OUT]
		self._pins["vel-fb"] = ["velocity", halcore.PIN_OUT]
		self._pins["index-enable"] = ["index-enable", halcore.PIN_BIDIR]
		self._pins["pos-rawcount"] = ["rawcount", halcore.PIN_OUT]
		"""
		self._pins[""] = ["count", PIN_OUT]
		self._pins[""] = ["reset", PIN_IN]
		"""

	def setup_params(self, jointno):
		self._params = []
		self._params.append(["scale", "[AXIS_%d]ENCODER_SCALE" % jointno])
		self._params.append(["index-invert", "0"])
		self._params.append(["index-mask", "0"])
		self._params.append(["index-mask-invert", "0"])
		self._params.append(["counter-mode", "0"])
		self._params.append(["filter", "1"])
		#self._params.append(["vel-timeout", "?"])

	def get_halname(self):
		mesa_fpga = self._mesa_db.get_mesa_fpga()
		index = mesa_fpga.get_enc_start_idx(self._mesa_db)
		index += self._index
		return mesa_fpga.get_halname() + ".encoder." + format(index, "02d")


class Resolver(MesaHALComp):
	def __init__(self, mesa_db, index):
		MesaHALComp.__init__(self, index)
		self._mesa_db = mesa_db

		#self._pins[""] = ["angle", halcore.PIN_OUT]
		self._pins["pos-fb"] = ["position", halcore.PIN_OUT]
		self._pins["vel-fb"] = ["velocity", halcore.PIN_OUT]
		#self._pins[""] = ["count", halcore.PIN_OUT]
		self._pins["pos-rawcount"] = ["rawcounts", halcore.PIN_OUT]
		#self._pins[""] = ["reset", halcore.PIN_IN]
		self._pins["index-enable"] = ["index-enable", halcore.PIN_BIDIR]
		#self._pins[""] = ["error", halcore.PIN_OUT]

	def setup_params(self, jointno):
		self._params = []
		self._params.append(["scale", "[AXIS_%d]ENCODER_SCALE" % jointno])
		#self._params.append(["velocity-scale", "0"])
		#self._params.append(["excitation-khz", "0"])

	def get_halname(self):
		mesa_fpga = self._mesa_db.get_mesa_fpga()
		index = mesa_fpga.get_enc_start_idx(self._mesa_db)
		index += self._index
		return mesa_fpga.get_halname() + ".resolver." + format(index, "02d")


class Input(halcore.HALComp):
	def __init__(self, sserial, index):
		halcore.HALComp.__init__(self)
		self._sserial = sserial
		self._index = index

	def get_halname(self):
		mesa_fpga = self._sserial.get_mesa_db().get_mesa_fpga()
		return mesa_fpga.get_halname() + "." + self._sserial.get_halname() + ".input-" + format(self._index, "02d")

	def get_pindir(self, netname):
		return halcore.PIN_OUT


class Output(halcore.HALComp):
	def __init__(self, sserial, index):
		halcore.HALComp.__init__(self)
		self._sserial = sserial
		self._index = index

	def get_halname(self):
		mesa_fpga = self._sserial.get_mesa_db().get_mesa_fpga()
		return mesa_fpga.get_halname() + "." + self._sserial.get_halname() + ".output-" + format(self._index, "02d")

	def get_pindir(self, netname):
		return halcore.PIN_IN
