#    This file is part of the LinuxCNC Machine Builder, a graphical
#    configuration editor for LinuxCNC. Copyright 2013 Arvid Brodin.
#
#    Although all the code is new, this program is influenced by
#    PNCconf (Copyright 2009 Chris Morley), in turn based on stepconf,
#    a graphical configuration editor for linuxcnc (Copyright 2007 Jeff
#    Epler).
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from hwic import hwic
from hardware import generic
from halcomp import hostmot2
from halcomp import halcore

class MesaFPGA(generic.Hardware):
	def __init__(self, platform, name):
		generic.Hardware.__init__(self, name)
		self._platform = platform
		self._name = name
		self.connectors = []
		self.daughters = []

	"""
	# Add daughterboard
	def add_daughter(self, db, conn):
		# FIXME: check conn is valid
		self.daughters.append([db, conn])

	def no_daughters(self):
		return len(self.daughters)
	def display_conns(self):
		for conn in self.connectors:
			conn.display(self.halname)

	def display_pins(self):
		for daughter in self.daughters:
			daughter[0].display(self.halname + ".")

	def generate_hal(self, file):
		print >>file, "setp\t" + self.halname + ".watchdog.timeout_ns 10000000"
		print >>file, "addf\t" + self.halname + ".read servo-thread"
		print >>file, "addf\t" + self.halname + ".write servo-thread"
		print >>file, "addf\t" + self.halname + ".pet_watchdog servo-thread"
	"""

	def get_halname(self):
		raise Exception("Abstract method called")

	def get_enc_start_idx(self, mesa_db):
		index = 0
		for hwic in self._hwic:
			if hwic.is_connected():
				card = hwic.get_connected()
				if card == mesa_db:
					return index
				index += card.get_encoder_count()
		raise Exception("Invalid parameter: Card not found")

	def get_sserial_start_idx(self, mesa_db):
		index = 0
		for hwic in self._hwic:
			if hwic.is_connected():
				card = hwic.get_connected()
			if card == mesa_db:
				return index
			index += card.get_sserial_count()
		raise Exception("Invalid parameter: Card not found")


class M5I23(MesaFPGA):
	UPCONN = hwic.Direct

	def __init__(self, platform):
		MesaFPGA.__init__(self, platform, "5I23")
		self._hwic.append(hwic.MesaP50(self, "P2"))
		self._hwic.append(hwic.MesaP50(self, "P3"))
		self._hwic.append(hwic.MesaP50(self, "P4"))

	def get_halname(self):
		return "hm2_5i25.%d" % self._platform.get_card_index(self)


class M5I25(MesaFPGA):
	UPCONN = hwic.Direct

	def __init__(self, platform):
		MesaFPGA.__init__(self, platform, "5I25")
		self._hwic.append(hwic.MesaP25(self, "P3"))
		self._hwic.append(hwic.MesaP25(self, "P2"))

	def get_halname(self):
		return "hm2_5i25.%d" % self._platform.get_card_index(self)


################ Daughtercards #################

# "Fake" owner object to handle sserial hal names
class Sserial(object):
	def __init__(self, mesa_db, index, name):
		self._mesa_db = mesa_db		# Mesa daughter board
		self._index = index
		self._name = name

	def get_mesa_db(self):
		return self._mesa_db

	def get_halname(self):
		mesa_fpga = self._mesa_db.get_mesa_fpga();
		index = mesa_fpga.get_sserial_start_idx(self._mesa_db)
		index += self._index
		return mesa_fpga.get_halname() + "." + self._name + "." + str(index//8) + "." + str(index % 8)


class MesaDaughter(generic.Hardware):
	def __init__(self, mesa_fpga, name, enc_cnt, sser_cnt):
		generic.Hardware.__init__(self, name)
		self._mesa_fpga = mesa_fpga
		self._encoder_count = enc_cnt
		self._sserial_count = sser_cnt

	def get_mesa_fpga(self):
		return self._mesa_fpga

	# Used by owning FPGA card to figure out the starting indices
	# on each port.
	def get_encoder_count(self):
		return self._encoder_count

	def get_sserial_count(self):
		return self._sserial_count


class M7I77(MesaDaughter):
	UPCONN = hwic.MesaP25

	def __init__(self, mesa_fpga):
		MesaDaughter.__init__(self, mesa_fpga, "7I77", 6, 3)

		self._hwic.append(hwic.MesaRS422(self, "TB6"))

		sserial0 = Sserial(self, 0, "7i77")	# SSerial 7i77.0.0
		sserial1 = Sserial(self, 1, "7i77")	# SSerial 7i77.0.1

		for i in range(3):
			self._terminals.append(generic.TermFeedback(
				"TB3-ENC" + str(i),
				hostmot2.Encoder(self, i)))
		for i in range(3, 6):
			self._terminals.append(generic.TermFeedback(
				"TB4-ENC" + str(i),
				hostmot2.Encoder(self, i)))
		for i in range(6):
			self._terminals.append(generic.TermControl(
				"TB5-AOUT" + str(i),
				hostmot2.AnalogOut(sserial1, i)))
		for i in range(16):
			self._terminals.append(generic.TermInput(
				"TB8-INPUT" + str(i),
				hostmot2.Input(sserial0, i)))
		for i in range(8):
			self._terminals.append(generic.TermOutput(
				"TB8-OUTPUT" + str(i),
				hostmot2.Output(sserial0, i)))
		for i in range(16, 32):
			self._terminals.append(generic.TermInput(
				"TB7-INPUT" + format(i, "01d"),
				hostmot2.Input(sserial0, i)))
		for i in range(8, 16):
			self._terminals.append(generic.TermOutput(
				"TB7-OUTPUT" + format(i, "01d"),
				hostmot2.Output(sserial0, i)))


################ 50-p Daughtercards #################


class M7I44(MesaDaughter):
	UPCONN = hwic.MesaP50

	def __init__(self, mesa_fpga):
		MesaDaughter.__init__(self, mesa_fpga, "7I44", 0, 0)

		for i in range(8):
			self._hwic.append(hwic.MesaRS422(self, "CHAN" + str(i)))


class M7I49(MesaDaughter):
	UPCONN = hwic.MesaP50

	def __init__(self, mesa_fpga):
		MesaDaughter.__init__(self, mesa_fpga, "7I49", 0, 0)

		sserial0 = Sserial(self, 0, "7i49")

		for i in range(2):
			self._terminals.append(generic.TermControl(
				"P4-AOUT" + str(i),
				hostmot2.AnalogOut(sserial0, i)))
		for i in range(2, 4):
			self._terminals.append(generic.TermControl(
				"P3-AOUT" + str(i),
				hostmot2.AnalogOut(sserial0, i)))
		for i in range(4, 6):
			self._terminals.append(generic.TermControl(
				"P2-AOUT" + str(i),
				hostmot2.AnalogOut(sserial0, i)))

		for i in range(2):
			self._terminals.append(generic.TermFeedback(
				"P4-RESOLVER" + str(i),
				hostmot2.Resolver(self, i)))
		for i in range(2, 4):
			self._terminals.append(generic.TermFeedback(
				"P3-RESOLVER" + str(i),
				hostmot2.Resolver(self, i)))
		for i in range(4, 6):
			self._terminals.append(generic.TermFeedback(
				"P2-RESOLVER" + str(i),
				hostmot2.Resolver(self, i)))



################ Sserial Daughtercards #################

class M7I70(MesaDaughter):
	UPCONN = hwic.MesaRS422

	def __init__(self, mesa_fpga, sserial):
		MesaDaughter.__init__(self, mesa_fpga, "7I70", 0, 0)

		for i in range(24):
			self._terminals.append(generic.TermInput(
				"TB3-INPUT" + format(i, "01d"),
				hostmot2.Input(sserial, i)))
		for i in range(24, 48):
			self._terminals.append(generic.TermInput(
				"TB2-INPUT" + format(i, "01d"),
				hostmot2.Input(sserial, i)))


class M7I71(MesaDaughter):
	UPCONN = hwic.MesaRS422

	def __init__(self, mesa_fpga, sserial):
		MesaDaughter.__init__(self, mesa_fpga, "7I71", 0, 0)

		for i in range(24):
			self._terminals.append(generic.TermOutput(
				"TB3-OUTPUT" + format(i, "01d"),
				hostmot2.Output(sserial, i)))
		for i in range(24, 48):
			self._terminals.append(generic.TermOutput(
				"TB2-OUTPUT" + format(i, "01d"),
				hostmot2.Output(sserial, i)))
